/*****************************************************************************/
/*									     */
/*									     */
/*	Xpack version 1.00 -- module X-events.c				     */
/*									     */
/*	Event handler functions for the X window system.		     */
/*	Written by Michael Bischoff (mbi@mo.math.nat.tu-bs.de)		     */
/*	November-1994							     */
/*	see COPYRIGHT.xpack for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/
#include "X-sok.h"
#include <X11/keysym.h>		/* X11 key code definitions */

void refresh_screen(void) {
    XClearArea(dpy, table, 0, 0, 0, 0, True);
}

/* event entry points are: key_press, button_press, button_release, redraw_table */

extern int offx, offy;	/* mouse coors */

static int take_tile(int mx, int my) {
    int x = mx / DX, y = my / DY, k;
    if ((k = P.board[y][x]) < 1) {
	show_message("No tile at (%d,%d)", x, y);
	return 0;
    }
    activate_and_redraw(k);
    popup_active((P.position[k].x+1)*DX, (P.position[k].y+1)*DY, mx, my);
    show_message("Tile %s taken at (%d,%d)",
		 P.t[active->tilenr].name,
		 P.position[k].x, P.position[k].y);
    return 1;
}

static void drop_tile(int mx, int my) {
    if (mx >= DX && my >= DY &&
	mx < DX*(P.t[0].width+1) &&
	my < DY*(P.t[0].height+1)) {
	int x, y;
	x = (mx - offx - DX/2) / DX;
	y = (my - offy - DY/2) / DY;
	if (place_and_redraw(x, y)) {
	    show_message("Tile placed");
	} else {
	    show_message("Doesn't fit");
	    if (active->x >= 0) {
		set_type(active->type);		/* old type */
		place_and_redraw(active->x, active->y);
	    } else
		discard_active();
	}
    } else {
	discard_active();
	
    }
    popdown_tile();
    state = 0;
}

void button_press(XButtonPressedEvent *xev) {
    if (state != 0 || xev->button != Button2)
	return;
    if (take_tile(xev->x, xev->y))
	state = 1;
}

int place_and_redraw(int x, int y) {
    if (place_active(x, y)) {
	doPaint(x+1, y+1, x+a.w, y+a.h);
	return 1;
    }
    return 0;
}

void activate_and_redraw(int k) {
    int x, y;
    x = P.position[k].x;
    y = P.position[k].y;
    activate_used(k);
    doPaint(x+1, y+1, x+a.w, y+a.h);
}

void button_release(XButtonPressedEvent *xev) {
    int x, y;
    x = xev->x / DX;	/* destination with offset 1 */
    y = xev->y / DY;
    switch (state) {
    case 0:	/* no active tile */
	switch (xev->button) {
	case Button1:            /* select tile */
	    if (take_tile(xev->x, xev->y))
		state = 2;
	    break;
	case Button3:
	    {   char msg[200];
		struct position *pp = P.position + P.board[y][x];
		if (pp > P.position) {
		    sprintf(msg, "Tile %s",
			    P.t[pp->tilenr].name);
		    if (pp->type & 3)
			sprintf(msg+strlen(msg), ", rotation %d",
				(pp->type & 3) * 90);
		    if (pp->type & 4)
			strcat(msg, ", mirrored");
		    show_message(msg);
		} else
		    show_message("No tile");
	    }
	    break;
	}
	return;
    case 1:		/* dragging */
	switch (xev->button) {
	case Button2:            /* place tile */
	    drop_tile(xev->x, xev->y);
	    return;
	}
	return;
    case 2:	/* active tile selected */
	switch (xev->button) {
	case Button1:
	    drop_tile(xev->x, xev->y);
	    return;
	case Button2:            /* mirror tile */
	    /* always mirror olong the vertical axis, so the rotation may change */
	    if (P.t[a.tilenr].type & ALLOW_MIRROR) {
		set_type(a.type & 1 ? a.type^6 : a.type^4);
		show_message("OK");
		resize_tile();
	    } else
		show_message("Not allowed or effectless for this tile");
	    return;
	case Button3:            /* rotate tile */
	    if (P.t[a.tilenr].type & ALLOW_90)
		set_type((a.type & ~3) | ((a.type+1) & 3));
	    else if (P.t[a.tilenr].type & ALLOW_180)
		set_type(a.type ^ ALLOW_180);
	    else {
		show_message("Not allowed or effectless for this tile");
		return;
	    }
	    show_message("OK");
	    resize_tile();
	    return;
	}
	return;
    }
}







void key_press(XKeyPressedEvent *xev) {
    char str[32];
    int num;

#define	get_name_field()	get_selection()

    num = XKeycodeToKeysym(dpy, xev->keycode, 0);

    if (num & 0xff00) {
    	switch (num) {
    	case XK_Up:
	    key_pressed("Up");
	    return;
    	case XK_Left:
	    key_pressed("Left");
	    return;
    	case XK_Down:
	    key_pressed("Down");
	    return;
    	case XK_Right:
	    key_pressed("Right");
	    return;
    	case XK_Return:
    	case XK_Linefeed:
	    key_pressed("\n");
	    return;
    	case XK_BackSpace:
    	case XK_Delete:
	    key_pressed("\b");
	    return;
    	case XK_Escape:
	    key_pressed("\033");
	    return;
	}
        return;
    }

    num = XLookupString(xev, str, 31, NULL, NULL);
    if (num == 0)
	return;
    str[num] = '\0';		/* NULL to terminate it */

    key_pressed(str);
}

/*****************************************************************************/
/*									     */
/*	Functions for resize events and resize requests			     */
/*									     */
/*****************************************************************************/

/* 1) hard resizes (i.e. forcing the outer window to change size) */
/*    I think these are not liked in the Xaw community */

void cmd_Resize(void) {
    XSize_t w, h;
    w = graphic.width;
    h = graphic.height;
    Force_Resize(w, h);
}

/* event handler function. This function is called by the Widget in response
   to a request from us. In Xaw, this is a resize of the logical area, i.e.
   of the virtual size of the tableau. */

void resize_event(XSize_t w, XSize_t h) {
#ifdef LABER
    printf("resize event to (%d,%d) called\n", w, h);
#endif
    if (graphic.height == h && graphic.width == w)
	return;		/* no change of size */

    /* in xlib, we must clear the new area by hand; there may be illegal data
       left in the server. This applies to Xaw as well */
    {   XExposeEvent xev;
	xev.count = -1;
        if (gamegraphic) {
	    if (graphic.height < h) {
		/* window is greater now */
		XClearArea(dpy, table, 0, graphic.height, graphic.width, h - graphic.height, True);
		++xev.count;
	    }
	    if (graphic.width < w) {
		/* window is greater now */
		XClearArea(dpy, table, graphic.width, 0, w - graphic.width, h, True);
		++xev.count;
	    }
	    if (xev.count >= 0) {
		/* generate synthetic expose events for the new area */
		/* this must be done before we possibly change the layout */
		if (graphic.height < h) {
		    /* window is greater now */
		    xev.x = 0;
		    xev.y = graphic.height;
		    xev.width = graphic.width;
		    xev.height = h - graphic.height;
		    redraw_table(&xev);
		    --xev.count;
		}
		if (graphic.width < w) {
		    /* window is greater now */
		    xev.x = graphic.width;
		    xev.y = 0;
		    xev.width = w - graphic.width;
		    xev.height = h;
		    redraw_table(&xev);
		}
	    }
	}
    }
    graphic.height = h;
    graphic.width = w;

    if (!gamegraphic)
	return;
}

